@@ -24,6 +24,11 @@ import java.util.concurrent.ConcurrentHashMap
 import java.util.concurrent.CopyOnWriteArraySet
 import java.util.concurrent.atomic.AtomicInteger
 
+private data class VisitResult(val download: Boolean = false, val children: Set<SpongeUri> = setOf())
+
+private val skipVisitResult = VisitResult()
+private val downloadVisitResult = VisitResult(download = true)
+
 private val htmlMimeTypes = setOf(ContentType.TEXT_HTML.mimeType, ContentType.APPLICATION_XHTML_XML.mimeType)
 
 private val attributeKeys = mapOf(
@@ -47,7 +52,7 @@ private fun toSpongeUri(element: Element, attributeKey: String): SpongeUri? {
 class Sponge(private val spongeService: SpongeService, private val spongeConfig: SpongeConfig) {
     private val requestContext = newFixedThreadPoolContext(spongeConfig.concurrentRequests, "request")
     private val downloadContext = newFixedThreadPoolContext(spongeConfig.concurrentDownloads, "download")
-    private val spongeUris = ConcurrentHashMap<String, SpongeUri>()
+    private val visitResults = ConcurrentHashMap<SpongeUri, VisitResult>()
     private val downloadedUris = CopyOnWriteArraySet<SpongeUri>()
     private val visitedCount = AtomicInteger()
 
@@ -57,55 +62,42 @@ class Sponge(private val spongeService: SpongeService, private val spongeConfig:
         if (visitedCount.incrementAndGet() > spongeConfig.maximumUris) return
 
         try {
-            spongeUris.computeIfAbsent(spongeUri.uri) {
-                initialize(spongeUri)
-            }.let {
-                downloadOrVisitChildren(it, parents)
+            val visitResult = getVisitResult(spongeUri)
+
+            if (visitResult !== skipVisitResult) {
+                downloadOrVisitChildren(visitResult, spongeUri, parents)
             }
         } catch (t: Throwable) {
             System.err.println("⚠ Processing failed for $spongeUri: ${t.rootMessage}")
         }
     }
 
-    private fun initialize(spongeUri: SpongeUri): SpongeUri {
+    private fun getVisitResult(spongeUri: SpongeUri) =
+        visitResults.computeIfAbsent(spongeUri) { createVisitResult(spongeUri) }
+
+    private fun createVisitResult(spongeUri: SpongeUri): VisitResult {
         val extension = FilenameUtils.getExtension(spongeUri.path)
+        var visitResult = skipVisitResult
 
         if (spongeConfig.fileExtensions.contains(extension)) {
-            spongeUri.download = true
+            visitResult = downloadVisitResult
         } else {
             val response = spongeService.request(spongeUri)
             val mimeType = ContentType.parse(response.contentType()).mimeType
 
             if (htmlMimeTypes.contains(mimeType)) {
                 val document = Jsoup.parse(response.body(), response.url().toExternalForm())
+                val children = getChildren(spongeUri, document)
 
-                spongeUri.children = getChildren(spongeUri, document)
+                if (children.isNotEmpty()) {
+                    visitResult = VisitResult(children = children)
+                }
             } else if (spongeConfig.mimeTypes.contains(mimeType)) {
-                spongeUri.download = true
+                visitResult = downloadVisitResult
             }
         }
 
-        return spongeUri
-    }
-
-    private suspend fun downloadOrVisitChildren(spongeUri: SpongeUri, parents: Set<SpongeUri>) {
-        if (spongeUri.download) {
-            download(spongeUri)
-        } else if (spongeUri.children.isNotEmpty() && parents.size < spongeConfig.maximumDepth) {
-            visit(spongeUri.children, parents + spongeUri)
-        }
-    }
-
-    private suspend fun download(spongeUri: SpongeUri) {
-        if (downloadedUris.add(spongeUri)) {
-            withContext(downloadContext) { spongeService.download(spongeUri) }
-        }
-    }
-
-    private suspend fun visit(spongeUris: Set<SpongeUri>, parents: Set<SpongeUri>) {
-        spongeUris.minus(parents)
-            .map { GlobalScope.async(requestContext) { visit(it, parents) } }
-            .awaitAll()
+        return visitResult
     }
 
     private fun getChildren(spongeUri: SpongeUri, document: Document): Set<SpongeUri> {
@@ -126,11 +118,35 @@ class Sponge(private val spongeService: SpongeService, private val spongeConfig:
             .asSequence()
             .distinct()
             .mapNotNull { toSpongeUri(it, attributeKey) }
-            .filter { it != spongeUri && isHostVisitable(it.host) && !downloadedUris.contains(it) }
+            .filter { it != spongeUri && isHostVisitable(it.host) }
     }
 
     private fun isHostVisitable(host: String): Boolean {
         return host == spongeConfig.spongeUri.host ||
             spongeConfig.includeSubdomains && host.endsWith(spongeConfig.spongeUri.host)
     }
+
+    private suspend fun downloadOrVisitChildren(
+        visitResult: VisitResult,
+        spongeUri: SpongeUri,
+        parents: Set<SpongeUri>
+    ) {
+        if (visitResult.download) {
+            download(spongeUri)
+        } else if (parents.size < spongeConfig.maximumDepth) {
+            visit(visitResult.children, parents + spongeUri)
+        }
+    }
+
+    private suspend fun download(spongeUri: SpongeUri) {
+        if (downloadedUris.add(spongeUri)) {
+            withContext(downloadContext) { spongeService.download(spongeUri) }
+        }
+    }
+
+    private suspend fun visit(spongeUris: Set<SpongeUri>, parents: Set<SpongeUri>) {
+        spongeUris.minus(parents)
+            .map { GlobalScope.async(requestContext) { visit(it, parents) } }
+            .awaitAll()
+    }
 }
